@@ -1,4 +1,4 @@
-# Copyright (c) 2015-2019 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2015-2022 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -26,7 +26,7 @@ import unittest
 import asyncssh
 
 from asyncssh.auth import MSG_USERAUTH_PK_OK, lookup_client_auth
-from asyncssh.auth import get_server_auth_methods, lookup_server_auth
+from asyncssh.auth import get_supported_server_auth_methods, lookup_server_auth
 from asyncssh.auth import MSG_USERAUTH_GSSAPI_RESPONSE
 from asyncssh.constants import MSG_USERAUTH_REQUEST, MSG_USERAUTH_FAILURE
 from asyncssh.constants import MSG_USERAUTH_SUCCESS
@@ -34,7 +34,7 @@ from asyncssh.gss import GSSClient, GSSServer
 from asyncssh.packet import SSHPacket, Boolean, Byte, NameList, String
 
 from .util import asynctest, gss_available, patch_gss
-from .util import AsyncTestCase, ConnectionStub
+from .util import AsyncTestCase, ConnectionStub, get_test_key
 
 
 class _AuthConnectionStub(ConnectionStub):
@@ -433,24 +433,27 @@ class _TestAuth(AsyncTestCase):
 
         with self.subTest('No auth methods'):
             server_conn = _AuthServerStub()
-            self.assertEqual(get_server_auth_methods(server_conn), [])
+            self.assertEqual(
+                get_supported_server_auth_methods(server_conn), [])
             server_conn.close()
 
         with self.subTest('All auth methods'):
             gss_host = '1' if gss_available else None
-            server_conn = _AuthServerStub(gss_host=gss_host,
-                                          host_based_auth=True,
-                                          public_key_auth=True,
-                                          password_auth=True, kbdint_auth=True)
+            server_conn = _AuthServerStub(
+                gss_host=gss_host, host_based_auth=True, public_key_auth=True,
+                password_auth=True, kbdint_auth=True)
+
             if gss_available: # pragma: no branch
-                self.assertEqual(get_server_auth_methods(server_conn),
-                                 [b'gssapi-keyex', b'gssapi-with-mic',
-                                  b'hostbased', b'publickey',
-                                  b'keyboard-interactive', b'password'])
+                self.assertEqual(
+                    get_supported_server_auth_methods(server_conn),
+                    [b'gssapi-keyex', b'gssapi-with-mic', b'hostbased',
+                     b'publickey', b'keyboard-interactive', b'password'])
             else: # pragma: no cover
-                self.assertEqual(get_server_auth_methods(server_conn),
-                                 [b'hostbased', b'publickey',
-                                  b'keyboard-interactive', b'password'])
+                self.assertEqual(
+                    get_supported_server_auth_methods(server_conn),
+                    [b'hostbased', b'publickey',
+                     b'keyboard-interactive', b'password'])
+
             server_conn.close()
 
         with self.subTest('Unknown auth method'):
@@ -514,7 +517,7 @@ class _TestAuth(AsyncTestCase):
     async def test_hostbased_auth(self):
         """Unit test host-based authentication"""
 
-        hkey = asyncssh.generate_private_key('ssh-rsa')
+        hkey = get_test_key('ssh-rsa')
         cert = hkey.generate_host_certificate(hkey, 'host')
 
         with self.subTest('Host-based auth not available'):
@@ -538,7 +541,7 @@ class _TestAuth(AsyncTestCase):
     async def test_publickey_auth(self):
         """Unit test public key authentication"""
 
-        ckey = asyncssh.generate_private_key('ssh-rsa')
+        ckey = get_test_key('ssh-rsa')
         cert = ckey.generate_user_certificate(ckey, 'name')
 
         with self.subTest('Public key auth not available'):
