@@ -1,4 +1,4 @@
-# Copyright (c) 2016-2020 by Ron Frederick <ronf@timeheart.net> and others.
+# Copyright (c) 2016-2022 by Ron Frederick <ronf@timeheart.net> and others.
 #
 # This program and the accompanying materials are made available under
 # the terms of the Eclipse Public License v2.0 which accompanies this
@@ -205,6 +205,18 @@ class _UNIXConnectionServer(Server):
             return listen_path != 'fail'
 
 
+class _UNIXAsyncConnectionServer(_UNIXConnectionServer):
+    """Server for testing async direct and forwarded UNIX connections"""
+
+    async def unix_server_requested(self, listen_path):
+        """Handle a request to create a new UNIX domain listener"""
+
+        if listen_path == 'open':
+            return _EchoPathListener(self._conn)
+        else:
+            return listen_path != 'fail'
+
+
 class _CheckForwarding(ServerTestCase):
     """Utility functions for AsyncSSH forwarding unit tests"""
 
@@ -246,6 +258,24 @@ class _CheckForwarding(ServerTestCase):
 
         self.assertEqual(b''.join(data), result)
 
+    async def _check_local_connection(self, listen_port, delay=None):
+        """Open a local connection and test if an input line is echoed back"""
+
+        reader, writer = await asyncio.open_connection('127.0.0.1',
+                                                       listen_port)
+
+        await self._check_echo_line(reader, writer, delay=delay)
+
+    async def _check_local_unix_connection(self, listen_path):
+        """Open a local connection and test if an input line is echoed back"""
+
+        # pylint doesn't think open_unix_connection exists
+        # pylint: disable=no-member
+        reader, writer = await asyncio.open_unix_connection(listen_path)
+        # pylint: enable=no-member
+
+        await self._check_echo_line(reader, writer)
+
 
 class _TestTCPForwarding(_CheckForwarding):
     """Unit tests for AsyncSSH TCP connection forwarding"""
@@ -266,13 +296,6 @@ class _TestTCPForwarding(_CheckForwarding):
 
         await self._check_echo_block(reader, writer)
 
-    async def _check_local_connection(self, listen_port, delay=None):
-        """Open a local connection and test if an input line is echoed back"""
-
-        reader, writer = await asyncio.open_connection(None, listen_port)
-
-        await self._check_echo_line(reader, writer, delay=delay)
-
     @asynctest
     async def test_ssh_create_tunnel(self):
         """Test creating a tunneled SSH connection"""
@@ -306,7 +329,7 @@ class _TestTCPForwarding(_CheckForwarding):
         """Test failed connection on a tunneled SSH connection via string"""
 
         with self.assertRaises(asyncssh.ChannelOpenError):
-            await asyncssh.connect('0.0.0.1',
+            await asyncssh.connect('\xff',
                                    tunnel='%s:%d' % (self._server_addr,
                                                      self._server_port))
 
@@ -413,7 +436,7 @@ class _TestTCPForwarding(_CheckForwarding):
         """Test open failure on a tunneled SSH listener via string"""
 
         with self.assertRaises(asyncssh.ChannelListenError):
-            await asyncssh.listen('0.0.0.1',
+            await asyncssh.listen('\xff',
                                   tunnel='%s:%d' % (self._server_addr,
                                                     self._server_port),
                                   server_factory=Server,
@@ -571,6 +594,32 @@ class _TestTCPForwarding(_CheckForwarding):
                 await self._check_local_connection(listener.get_port(),
                                                    delay=0.1)
 
+    @unittest.skipIf(sys.platform == 'win32',
+                     'skip UNIX domain socket tests on Windows')
+    @asynctest
+    async def test_forward_local_path_to_port(self):
+        """Test forwarding of a local UNIX domain path to a remote TCP port"""
+
+        async with self.connect() as conn:
+            async with conn.forward_local_path_to_port('local', '', 7):
+                await self._check_local_unix_connection('local')
+
+        os.remove('local')
+
+    @unittest.skipIf(sys.platform == 'win32',
+                     'skip UNIX domain socket tests on Windows')
+    @asynctest
+    async def test_forward_local_path_to_port_failure(self):
+        """Test failure forwarding a local UNIX domain path to a TCP port"""
+
+        open('local', 'w').close()
+
+        async with self.connect() as conn:
+            with self.assertRaises(OSError):
+                await conn.forward_local_path_to_port('local', '', 7)
+
+        os.remove('local')
+
     @asynctest
     async def test_forward_local_port_pause(self):
         """Test pause during forwarding of a local port"""
@@ -579,7 +628,7 @@ class _TestTCPForwarding(_CheckForwarding):
             async with conn.forward_local_port('', 0, '', 8) as listener:
                 listen_port = listener.get_port()
 
-                reader, writer = await asyncio.open_connection(None,
+                reader, writer = await asyncio.open_connection('127.0.0.1',
                                                                listen_port)
 
                 writer.write(4*1024*1024*b'\0')
@@ -589,7 +638,6 @@ class _TestTCPForwarding(_CheckForwarding):
                 writer.close()
                 await maybe_wait_closed(writer)
 
-
     @asynctest
     async def test_forward_local_port_failure(self):
         """Test failure in forwarding a local port"""
@@ -598,7 +646,7 @@ class _TestTCPForwarding(_CheckForwarding):
             async with conn.forward_local_port('', 0, '', 65535) as listener:
                 listen_port = listener.get_port()
 
-                reader, writer = await asyncio.open_connection(None,
+                reader, writer = await asyncio.open_connection('127.0.0.1',
                                                                listen_port)
 
                 self.assertEqual((await reader.read()), b'')
@@ -615,7 +663,7 @@ class _TestTCPForwarding(_CheckForwarding):
         async with self.connect() as conn:
             async with conn.forward_local_port('0.0.0.0', 0, '', 7) as listener:
                 with self.assertRaises(OSError):
-                    await conn.forward_local_port(None, listener.get_port(),
+                    await conn.forward_local_port('', listener.get_port(),
                                                   '', 7)
 
     @unittest.skipIf(sys.platform == 'win32',
@@ -627,9 +675,21 @@ class _TestTCPForwarding(_CheckForwarding):
         async with self.connect() as conn:
             async with conn.forward_local_port('::', 0, '', 7) as listener:
                 with self.assertRaises(OSError):
-                    await conn.forward_local_port(None, listener.get_port(),
+                    await conn.forward_local_port('', listener.get_port(),
                                                   '', 7)
 
+    @unittest.skipIf(sys.platform == 'win32',
+                     'skip UNIX domain socket tests on Windows')
+    @asynctest
+    async def test_forward_port_to_path_bind_error(self):
+        """Test error binding a local port forwarding to remote path"""
+
+        async with self.connect() as conn:
+            async with conn.forward_local_port('0.0.0.0', 0, '', 7) as listener:
+                with self.assertRaises(OSError):
+                    await conn.forward_local_port_to_path(
+                            '', listener.get_port(), '')
+
     @asynctest
     async def test_forward_connect_error(self):
         """Test error connecting a local forwarding port"""
@@ -638,9 +698,8 @@ class _TestTCPForwarding(_CheckForwarding):
             async with conn.forward_local_port('', 0, '', 1) as listener:
                 listen_port = listener.get_port()
 
-                reader, writer = await asyncio.open_connection(None,
+                reader, writer = await asyncio.open_connection('127.0.0.1',
                                                                listen_port)
-
                 self.assertEqual((await reader.read()), b'')
 
                 writer.close()
@@ -654,7 +713,8 @@ class _TestTCPForwarding(_CheckForwarding):
             async with conn.forward_local_port('', 0, '', 7) as listener:
                 listen_port = listener.get_port()
 
-                _, writer = await asyncio.open_connection(None, listen_port)
+                _, writer = await asyncio.open_connection('127.0.0.1',
+                                                          listen_port)
 
                 writer.close()
                 await maybe_wait_closed(writer)
@@ -669,13 +729,30 @@ class _TestTCPForwarding(_CheckForwarding):
         server_port = server.sockets[0].getsockname()[1]
 
         async with self.connect() as conn:
-            async with conn.forward_remote_port('', 0,
-                                                '', server_port) as listener:
+            async with conn.forward_remote_port(
+                    '', 0, '127.0.0.1', server_port) as listener:
                 await self._check_local_connection(listener.get_port())
 
         server.close()
         await server.wait_closed()
 
+    @unittest.skipIf(sys.platform == 'win32',
+                     'skip UNIX domain socket tests on Windows')
+    @asynctest
+    async def test_forward_remote_port_to_path(self):
+        """Test forwarding of a remote port to a local UNIX domain socket"""
+
+        server = await asyncio.start_unix_server(echo, 'local')
+
+        async with self.connect() as conn:
+            async with conn.forward_remote_port_to_path(
+                    '', 0, 'local') as listener:
+                await self._check_local_connection(listener.get_port())
+
+        server.close()
+        await server.wait_closed()
+        os.remove('local')
+
     @asynctest
     async def test_forward_remote_specific_port(self):
         """Test forwarding of a specific remote port"""
@@ -690,8 +767,8 @@ class _TestTCPForwarding(_CheckForwarding):
         sock.close()
 
         async with self.connect() as conn:
-            async with conn.forward_remote_port('', remote_port,
-                                                '', server_port) as listener:
+            async with conn.forward_remote_port(
+                    '', remote_port, '127.0.0.1', server_port) as listener:
                 await self._check_local_connection(listener.get_port())
 
         server.close()
@@ -810,16 +887,6 @@ class _TestUNIXForwarding(_CheckForwarding):
 
         await self._check_echo_line(reader, writer, encoded=True)
 
-    async def _check_local_unix_connection(self, listen_path):
-        """Open a local connection and test if an input line is echoed back"""
-
-        # pylint doesn't think open_unix_connection exists
-        # pylint: disable=no-member
-        reader, writer = await asyncio.open_unix_connection(listen_path)
-        # pylint: enable=no-member
-
-        await self._check_echo_line(reader, writer)
-
     @asynctest
     async def test_unix_connection(self):
         """Test opening a remote UNIX connection"""
@@ -932,6 +999,31 @@ class _TestUNIXForwarding(_CheckForwarding):
 
         os.remove('local')
 
+    @asynctest
+    async def test_forward_local_port_to_path(self):
+        """Test forwarding of a local port to a remote UNIX domain socket"""
+
+        async with self.connect() as conn:
+            async with conn.forward_local_port_to_path('', 0,
+                                                       '/echo') as listener:
+                await self._check_local_connection(listener.get_port(),
+                                                   delay=0.1)
+
+    @asynctest
+    async def test_forward_specific_local_port_to_path(self):
+        """Test forwarding of a specific local port to a UNIX domain socket"""
+
+        sock = socket.socket()
+        sock.bind(('', 0))
+        listen_port = sock.getsockname()[1]
+        sock.close()
+
+        async with self.connect() as conn:
+            async with conn.forward_local_port_to_path(
+                    '', listen_port, '/echo') as listener:
+                await self._check_local_connection(listener.get_port(),
+                                                   delay=0.1)
+
     @asynctest
     async def test_forward_remote_path(self):
         """Test forwarding of a remote UNIX domain path"""
@@ -953,6 +1045,26 @@ class _TestUNIXForwarding(_CheckForwarding):
         os.remove('echo')
         os.remove('local')
 
+    @asynctest
+    async def test_forward_remote_path_to_port(self):
+        """Test forwarding of a remote UNIX domain path to a local TCP port"""
+
+        server = await asyncio.start_server(echo, None, 0,
+                                            family=socket.AF_INET)
+        server_port = server.sockets[0].getsockname()[1]
+
+        path = os.path.abspath('echo')
+
+        async with self.connect() as conn:
+            async with conn.forward_remote_path_to_port(
+                    path, '127.0.0.1', server_port):
+                await self._check_local_unix_connection('echo')
+
+        os.remove('echo')
+
+        server.close()
+        await server.wait_closed()
+
     @asynctest
     async def test_forward_remote_path_failure(self):
         """Test failure of forwarding a remote UNIX domain path"""
@@ -1001,6 +1113,18 @@ class _TestUNIXForwarding(_CheckForwarding):
                 self.assertEqual(pkttype, asyncssh.MSG_REQUEST_FAILURE)
 
 
+class _TestAsyncUNIXForwarding(_TestUNIXForwarding):
+    """Unit tests for AsyncSSH UNIX connection forwarding with async return"""
+
+    @classmethod
+    async def start_server(cls):
+        """Start an SSH server which supports UNIX connection forwarding"""
+
+        return await cls.create_server(
+            _UNIXAsyncConnectionServer,
+            authorized_client_keys='authorized_keys')
+
+
 class _TestSOCKSForwarding(_CheckForwarding):
     """Unit tests for AsyncSSH SOCKS dynamic port forwarding"""
 
@@ -1072,7 +1196,8 @@ class _TestSOCKSForwarding(_CheckForwarding):
         with self.subTest(msg=msg, data=data):
             data = codecs.decode(data, 'hex')
 
-            reader, writer = await asyncio.open_connection(None, listen_port)
+            reader, writer = await asyncio.open_connection('127.0.0.1',
+                                                           listen_port)
 
             try:
                 await handler(reader, writer, data, *args)
@@ -1142,7 +1267,7 @@ class _TestSOCKSForwarding(_CheckForwarding):
         sock.close()
 
         async with self.connect() as conn:
-            async with conn.forward_socks(None, listen_port):
+            async with conn.forward_socks('', listen_port):
                 pass
 
     @unittest.skipIf(sys.platform == 'win32',
@@ -1152,6 +1277,6 @@ class _TestSOCKSForwarding(_CheckForwarding):
         """Test error binding a local dynamic forwarding port"""
 
         async with self.connect() as conn:
-            async with conn.forward_socks(None, 0) as listener:
+            async with conn.forward_socks('', 0) as listener:
                 with self.assertRaises(OSError):
-                    await conn.forward_socks(None, listener.get_port())
+                    await conn.forward_socks('', listener.get_port())
